<?php
namespace Inti\Db\Connection;

use Doctrine\DBAL\Configuration;
use Doctrine\DBAL\Driver\Connection;
use Doctrine\DBAL\DriverManager;
use Doctrine\DBAL\Exception as DbalException;
use Interop\Container\ContainerInterface;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Class ConnectionAbstractFactory
 *
 * @package Inti\Db\Connection
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-db
 */
final class ConnectionFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param null|array $options
     *
     * @return Connection
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $params = $this->getConfig($container, $requestedName);
        $config = new Configuration();

        try {
            return DriverManager::getConnection($params, $config);
        } catch (DbalException $e) {
            throw new ServiceNotCreatedException('Could not create a database connection.', 0, $e);
        }
    }

    /**
     * @param ContainerInterface $container
     * @param string $name
     * @return array|null
     */
    private function getConfig(ContainerInterface $container, string $name): ?array
    {
        $configs = $container->get('config')[ConnectionManager::CONFIG_KEY]['connections'] ?? [];
        return $configs[$name] ?? null;
    }
}
